@@ -52,24 +52,6 @@ class Teacher(cfg.ConfigUtils, QstatusDefs):
         # The file name, not a file object
         self.m_lessonfile = None
         self.m_question = None
-    def get_instrument_config(self, num):
-        if self.get_bool('config/override_default_instrument'):
-            instr_low = self.get_int('config/lowest_instrument')
-            instr_low_vel = self.get_int('config/lowest_instrument_velocity')
-            instr_middle = self.get_int('config/middle_instrument')
-            instr_middle_vel = self.get_int('config/middle_instrument_velocity')
-            instr_high = self.get_int('config/highest_instrument')
-            instr_high_vel = self.get_int('config/highest_instrument_velocity')
-        else:
-            instr_low = instr_middle = instr_high \
-                        = self.get_int('config/preferred_instrument')
-            instr_low_vel = instr_middle_vel = instr_high_vel \
-                         = self.get_int('config/preferred_instrument_velocity')
-        if num == 2:
-            return instr_low, instr_low_vel, instr_high, instr_high_vel
-        else:
-            return instr_low, instr_low_vel, \
-                instr_middle, instr_middle_vel, instr_high, instr_high_vel
     def maybe_auto_new_question(self):
         if self.get_bool('new_question_automatically'):
             if self.m_timeout_handle is None:
@@ -232,15 +214,12 @@ class MelodicIntervalTeacher(Teacher):
         if self.q_status == self.QSTATUS_NO:
             return
         t = self.m_tonika
-        m = soundcard.Track()
-        m.set_bpm(self.get_int('config/default_bpm'))
-        m.set_patch(self.get_int('config/preferred_instrument'))
-        m.note(4, self.m_tonika.semitone_pitch(),
-               self.get_int('config/preferred_instrument_velocity'))
+
+        m = utils.new_track()
+        m.note(4, self.m_tonika.semitone_pitch())
         for i in self.m_question:
             t = t + i
-            m.note(4, t.semitone_pitch(),
-                   self.get_int('config/preferred_instrument_velocity'))
+            m.note(4, t.semitone_pitch())
         soundcard.synth.play_track(m)
 
 
@@ -323,8 +302,9 @@ class RhythmAddOnClass:
         """
         # FIXME can we use lessonfile.Rhythm insted of this?
         score = mpd.parser.parse_to_score_object(rhythm)
-        track = score.get_midi_events_as_percussion(cfg.get_int('config/preferred_instrument_velocity'))[0]
+        track = score.get_midi_events_as_percussion()[0]
         track.prepend_bpm(self.get_int("bpm"))
+        track.prepend_volume(cfg.get_int('config/preferred_instrument_volume'))
         track.replace_note(mpd.notename_to_int("c"),
                            self.get_int("config/rhythm_perc"))
         track.replace_note(mpd.notename_to_int("d"),
@@ -560,6 +540,10 @@ class Gui(gtk.VBox, cfg.ConfigUtils, QstatusDefs):
             cleanup_function()
             self._mpd_exception(e, sourcefile, lineno)
             return True
+        elif isinstance(e, osutils.BinaryBaseException):
+            cleanup_function()
+            self.g_win.display_error_message2(e.msg1, e.msg2)
+            return True
         elif isinstance(e, osutils.OsUtilsException):
             cleanup_function()
             self.g_win.display_exception_message(e)
