@@ -16,6 +16,7 @@
 
 import buildinfo
 import errno, sys
+import subprocess
 import locale
 import tempfile
 import shutil
@@ -36,7 +37,7 @@ def check_rcfile():
     """See default.config for rcfileversion values, meanings and
     a description of how to add config variables.
     """
-    rcfileversion = 12
+    rcfileversion = 15
     if cfg.get_int("app/rcfileversion") > rcfileversion:
         cfg.drop_user_config()
         return
@@ -86,6 +87,26 @@ def check_rcfile():
             cfg.del_key("%s/rhythm_perc" % s)
     if cfg.get_int("app/rcfileversion") <= 12:
         cfg.del_key("sound/card_info")
+    if cfg.get_int("app/rcfileversion") <= 13:
+        cfg.del_key("config/lowest_instrument_velocity")
+        cfg.del_key("config/middle_instrument_velocity")
+        cfg.del_key("config/highest_instrument_velocity")
+        cfg.del_key("config/preferred_instrument_velocity")
+    if cfg.get_int("app/rcfileversion") <= 14:
+        # We have to split the midi_to_wav_cmd into two strings, and
+        # moving the options to *_options, so that midi_to_wav_cmd only
+        # have the name of the binary. This to allow binaries in dirs
+        # with spaces.
+        for k in ("midi_to_wav", "wav_to_mp3", "wav_to_ogg"):
+            v = cfg.get_string("app/%s_cmd" % k).split(" ")
+            cfg.set_string("app/%s_cmd" % k, v[0])
+            cfg.set_string("app/%s_cmd_options" % k, " ".join(v[1:]))
+    if cfg.get_int("app/rcfileversion") <= 15:
+        for k in ("midi", "wav", "mp3", "ogg"):
+            v = cfg.get_string("sound/%s_player" % k).split(" ")
+            cfg.set_string("sound/%s_player" % k, v[0])
+            cfg.set_string("sound/%s_player_options" % k,
+                " ".join(v[1:]))
     cfg.set_int("app/rcfileversion", rcfileversion)
     try:
         a = mpd.notename_to_int(cfg.get_string("user/lowest_pitch"))
@@ -105,7 +126,6 @@ class SolfegeApp(cfg.ConfigUtils):
         options -- command line options parsed by optparse
         """
         cfg.ConfigUtils.__init__(self, 'solfege-app')
-        self.locate_external_programs()
         lessonfile.MusicBaseClass.temp_dir = tempfile.mkdtemp(prefix="solfege-")
         os.environ['SOLFEGETEMPDIR'] = lessonfile.MusicBaseClass.temp_dir
         # test_mode is when we are running a test from the Tests menu
@@ -117,20 +137,6 @@ class SolfegeApp(cfg.ConfigUtils):
         self.lessonfile_manager = lessonfile_manager
         self.m_sound_init_exception = None
         self.setup_sound()
-    def locate_external_programs(self):
-        """
-        This method check if the user has configured the path to some
-        programs we need. Set the config variable if not.
-        This is useful the first time Solfege is run.
-        """
-        if not os.path.exists(cfg.get_string("programs/csound")):
-            v = osutils.find_progs(("csound", "csound.exe"))
-            if v:
-                self.set_string("programs/csound", v[0])
-        if not os.path.exists(cfg.get_string("programs/mma")):
-            v = osutils.find_mma_executables()
-            if v:
-                self.set_string("programs/mma", v[0])
     def setup_sound(self):
         if sys.platform == 'win32' and \
                     cfg.get_string("sound/type") == "sequencer-device":
@@ -205,10 +211,10 @@ It is also possible that the OS sound setup is incorrect.
  % str(e).decode(locale.getpreferredencoding(), 'replace'))
     def play_happy_sound(self):
         mpd.play_music(r"\staff\relative c'{c16 e g a}", 180, 8,
-               cfg.get_int('config/preferred_instrument_velocity'))
+               cfg.get_int('config/preferred_instrument_volume'))
     def play_sad_sound(self):
         mpd.play_music(r"\staff\relative c'{<c,,8 cis>", 80, 58,
-               cfg.get_int('config/preferred_instrument_velocity'))
+               cfg.get_int('config/preferred_instrument_volume'))
     def please_help_me(self):
         # If m_viewer == 'docviewer', then we see the welcome message, and
         # there is no help for that.
@@ -284,8 +290,10 @@ It is also possible that the OS sound setup is incorrect.
                     midifilename = "-".join(v[:-1])+"."+w[1]
                 else:
                     midifilename = urlobj.filename
-                run_external_program(self.get_string("sound/midi_player"),
-                        self.m_ui.box_dict['docviewer'].m_htmlwidget.m_document_wd, midifilename)
+
+                soundcard.play_mediafile("midi", os.path.join(
+                    self.m_ui.box_dict['docviewer'].m_htmlwidget.m_document_wd,
+                    midifilename))
             else:
                 self.m_ui.display_docfile(urlobj.filename, urlobj.anchor)
         elif urlobj.protocol == 'solfege':
@@ -440,20 +448,32 @@ It is also possible that the OS sound setup is incorrect.
                     """
                     Return False if we think the convert failed.
                     """
-                    filter_str = "app/%s_to_%s_cmd" % (from_format, to_format)
+                    app_cfg_name = "app/%s_to_%s_cmd" % (from_format, to_format)
                     if from_format == 'midi':
                         from_ext = 'mid'
                     else:
                         from_ext = from_format
                     to_ext = to_format
-                    if not cfg.get_string(filter_str):
-                        self.m_ui.display_error_message2("Config variable not defined", "The missing or empty variable was '%s'" % filter_str)
+                    if not cfg.get_string(app_cfg_name):
+                        self.m_ui.display_error_message2("Config variable not defined", "The missing or empty variable was '%s'" % app_cfg_name)
                         return False
                     try:
-                        s = cfg.get_string(filter_str) % {
-                            'in': os.path.join(export_dir, "track-%i.%s" % (track_idx, from_ext)),
-                            'out': os.path.join(export_dir, "track-%i.%s" % (track_idx, to_ext))}
-                        os.system(s)
+                        inout = {
+                            'in': os.path.join(export_dir,
+                                    "track-%i.%s" % (track_idx, from_ext)),
+                            'out': os.path.join(export_dir,
+                                    "track-%i.%s" % (track_idx, to_ext))}
+                        opts = cfg.get_string(app_cfg_name + '_options').split(" ")
+                        opts = [x % inout for x in opts]
+                        # For some reasong setting the executable arg does
+                        # not work for Python 2.5.2
+                        try:
+                            subprocess.call(
+                                [cfg.get_string(app_cfg_name)] + opts)
+                        except OSError, e:
+                            raise osutils.BinaryForMediaConvertorException(app_cfg_name,
+                                cfg.get_string(app_cfg_name), e)
+
                         if os.path.exists(os.path.join(export_dir, "track-%i.%s" % (track_idx, to_ext))):
                             os.remove(os.path.join(export_dir, "track-%i.%s" % (track_idx, from_ext)))
                         else:
