@@ -102,12 +102,12 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         page_vbox.pack_start(vbox, False)
         category_box.pack_start(gu.hig_label_widget(
             _("Count in:"),
-            gu.PercussionNameComboBoxEntry("config", "countin_perc", "Claves"),
-            sizegroup))
+            gu.PercussionNameButton("config", "countin_perc", "Claves"),
+            sizegroup, True, True))
         category_box.pack_start(gu.hig_label_widget(
             _("Rhythm:"),
-            gu.PercussionNameComboBoxEntry("config", "rhythm_perc", "Side Stick"),
-            sizegroup))
+            gu.PercussionNameButton("config", "rhythm_perc", "Side Stick"),
+            sizegroup, True, True))
         ########
         # user #
         ########
@@ -158,38 +158,77 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
 
         # midi_to_wav
-        liststore = gtk.ListStore(gobject.TYPE_STRING)
-        liststore.append(("/usr/bin/timidity -Ow %(in)s -o %(out)s",))
-        self.g_wav_convertor = gtk.ComboBoxEntry(liststore)
-        self.g_wav_convertor.child.set_text(
-            self.get_string("app/midi_to_wav_cmd"))
-        self.g_wav_convertor.connect('changed', 
-            lambda w: self.set_string('app/midi_to_wav_cmd', w.child.get_text()))
+        self.g_wav_convertor = gtk.ComboBoxEntry(
+            gtk.ListStore(gobject.TYPE_STRING))
+        self.g_wav_convertor.get_model().append(("timidity",))
+        #
+        self.g_wav_convertor_options = gtk.ComboBoxEntry(
+            gtk.ListStore(gobject.TYPE_STRING))
+        self.g_wav_convertor_options.get_model().append(("-Ow %(in)s -o %(out)s",))
+        #
         category_vbox.pack_start(
             gu.hig_label_widget(_("MIDI to WAV:"),
-            self.g_wav_convertor, sizegroup))
+               (self.g_wav_convertor, self.g_wav_convertor_options),
+               sizegroup, True, True))
+        #
+        self.g_wav_convertor.child.set_text(
+            self.get_string("app/midi_to_wav_cmd"))
+        self.g_wav_convertor_options.child.set_text(
+            self.get_string("app/midi_to_wav_cmd_options"))
+        self.g_wav_convertor.connect('changed',
+            lambda w: self.set_string('app/midi_to_wav_cmd',
+                w.child.get_text()))
+        self.g_wav_convertor_options.connect('changed',
+            lambda w: self.set_string('app/midi_to_wav_cmd_options',
+                w.child.get_text()))
         # wav_to_mp3
-        liststore = gtk.ListStore(gobject.TYPE_STRING)
-        liststore.append(("/usr/bin/lame %(in)s %(out)s",))
-        self.g_mp3_convertor = gtk.ComboBoxEntry(liststore)
-        self.g_mp3_convertor.child.set_text(
-            self.get_string("app/wav_to_mp3_cmd"))
-        self.g_mp3_convertor.connect('changed', 
-            lambda w: self.set_string('app/wav_to_mp3_cmd', w.child.get_text()))
+        self.g_mp3_convertor = gtk.ComboBoxEntry(
+            gtk.ListStore(gobject.TYPE_STRING))
+        self.g_mp3_convertor.get_model().append(("lame",))
+        #
+        self.g_mp3_convertor_options = gtk.ComboBoxEntry(
+            gtk.ListStore(gobject.TYPE_STRING))
+        self.g_mp3_convertor_options.get_model().append(("%(in)s %(out)s",))
+        #
         category_vbox.pack_start(
             gu.hig_label_widget(_("WAV to MP3:"),
-            self.g_mp3_convertor, sizegroup))
+            (self.g_mp3_convertor, self.g_mp3_convertor_options),
+            sizegroup, True, True))
+        #
+        self.g_mp3_convertor.child.set_text(
+            self.get_string("app/wav_to_mp3_cmd"))
+        self.g_mp3_convertor_options.child.set_text(
+            self.get_string("app/wav_to_mp3_cmd_options"))
+        self.g_mp3_convertor.connect('changed', 
+            lambda w: self.set_string('app/wav_to_mp3_cmd',
+                w.child.get_text()))
+        self.g_mp3_convertor_options.connect('changed', 
+            lambda w: self.set_string('app/wav_to_mp3_cmd_options',
+                w.child.get_text()))
         # wav_to_ogg
-        liststore = gtk.ListStore(gobject.TYPE_STRING)
-        liststore.append(("/usr/bin/oggenc %(in)s",))
-        self.g_ogg_convertor = gtk.ComboBoxEntry(liststore)
-        self.g_ogg_convertor.child.set_text(
-            self.get_string("app/wav_to_ogg_cmd"))
-        self.g_ogg_convertor.connect('changed', 
-            lambda w: self.set_string('app/wav_to_ogg_cmd', w.child.get_text()))
+        self.g_ogg_convertor = gtk.ComboBoxEntry(
+            gtk.ListStore(gobject.TYPE_STRING))
+        self.g_ogg_convertor.get_model().append(("oggenc",))
+        #
+        self.g_ogg_convertor_options = gtk.ComboBoxEntry(
+            gtk.ListStore(gobject.TYPE_STRING))
+        self.g_ogg_convertor_options.get_model().append(("%(in)s",))
+        #
         category_vbox.pack_start(
             gu.hig_label_widget(_("WAV to OGG:"),
-            self.g_ogg_convertor, sizegroup))
+            (self.g_ogg_convertor, self.g_ogg_convertor_options),
+            sizegroup, True, True))
+        #
+        self.g_ogg_convertor.child.set_text(
+            self.get_string("app/wav_to_ogg_cmd"))
+        self.g_ogg_convertor_options.child.set_text(
+            self.get_string("app/wav_to_ogg_cmd_options"))
+        self.g_ogg_convertor.connect('changed', 
+            lambda w: self.set_string('app/wav_to_ogg_cmd',
+                w.child.get_text()))
+        self.g_ogg_convertor_options.connect('changed', 
+            lambda w: self.set_string('app/wav_to_ogg_cmd_options',
+                w.child.get_text()))
 
         self.add_gui_for_external_programs(page_vbox)
         ########
@@ -207,16 +246,38 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         self.g_csound_exe = gtk.ComboBoxEntry(liststore)
         self.g_csound_exe.child.set_text(self.get_string("programs/csound"))
         self.g_csound_exe.connect('changed', lambda w: self.set_string('programs/csound', self.g_csound_exe.child.get_text()))
-        box = gu.hig_label_widget(_("CSound:"), self.g_csound_exe, sizegroup)
+        box = gu.hig_label_widget(_("CSound:"), self.g_csound_exe,
+            sizegroup, True, True)
         category_vbox.pack_start(box)
         # MMA
         liststore = gtk.ListStore(gobject.TYPE_STRING)
-        for p in osutils.find_mma_executables():
+        for p in osutils.find_mma_executables(cfg.get_list("app/win32_ignore_drives")):
             liststore.append((p,))
         self.g_mma_exe = gtk.ComboBoxEntry(liststore)
         self.g_mma_exe.child.set_text(self.get_string("programs/mma"))
         self.g_mma_exe.connect('changed', lambda w: self.set_string('programs/mma', self.g_mma_exe.child.get_text()))
-        box = gu.hig_label_widget(_("Musical MIDI Accompaniment:"), self.g_mma_exe, sizegroup)
+        box = gu.hig_label_widget(_("Musical MIDI Accompaniment:"),
+            self.g_mma_exe, sizegroup, True, True)
+        category_vbox.pack_start(box)
+        # lilypond-book
+        liststore = gtk.ListStore(gobject.TYPE_STRING)
+        for p in osutils.find_progs(('lilypond-book',)):
+            liststore.append((p,))
+        self.g_lilypond_book_exe = gtk.ComboBoxEntry(liststore)
+        self.g_lilypond_book_exe.child.set_text(self.get_string("programs/lilypond-book"))
+        self.g_lilypond_book_exe.connect('changed', lambda w: self.set_string('programs/lilypond-book', self.g_lilypond_book_exe.child.get_text()))
+        box = gu.hig_label_widget(_("Lilypond-book:"),
+            self.g_lilypond_book_exe, sizegroup, True, True)
+        category_vbox.pack_start(box)
+        # latex
+        liststore = gtk.ListStore(gobject.TYPE_STRING)
+        for p in osutils.find_progs(('latex',)):
+            liststore.append((p,))
+        self.g_latex_exe = gtk.ComboBoxEntry(liststore)
+        self.g_latex_exe.child.set_text(self.get_string("programs/latex"))
+        self.g_latex_exe.connect('changed', lambda w: self.set_string('programs/latex', self.g_latex_exe.child.get_text()))
+        box = gu.hig_label_widget(_("Latex:"),
+            self.g_latex_exe, sizegroup, True, True)
         category_vbox.pack_start(box)
         # Mail program
         liststore = gtk.ListStore(gobject.TYPE_STRING)
@@ -227,7 +288,8 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         self.g_mail_program = gtk.ComboBoxEntry(liststore)
         self.g_mail_program.child.set_text(self.get_string("config/mua"))
         self.g_mail_program.child.connect('changed', lambda w: self.set_string('config/mua', self.g_mail_program.child.get_text()))
-        box = gu.hig_label_widget(_("_Mail:"), self.g_mail_program, sizegroup)
+        box = gu.hig_label_widget(_("_Mail:"), self.g_mail_program, sizegroup,
+            True, True)
         category_vbox.pack_start(box)
 
         #######
@@ -320,9 +382,17 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
                  # testfile is a file in lesson-files/share
                  'testfile': 'fifth-small-220.00.wav',
                  'label': _("WAV:"),
-                 # players is a list of tuples. The tuple has to elements:
-                 # '/path/to/player', 'comandline'
-                 'players': [],
+                 # players is a list of tuples. The tuple has to or more 
+                 # items. The first is the binary, and the rest is possible
+                 # sets of command line options that might work.
+                 # '/path/to/player', 'comandline', '
+                 'players': [
+                        ('gst-launch', 'playbin uri=file://%s',
+                                       'filesrc location=%s ! wavparse ! alsasink'),
+                        ('play', ''),
+                        ('aplay', ''),
+                        ('esdplay', ''),
+                 ],
                 }
             ]
         else:
@@ -331,73 +401,86 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
             {'id': 'midi',
              'testfile': 'fanfare.midi',
              'label': _("MIDI:"),
-             'players': [],
+             'players': [
+                         ('gst-launch', 'playbin uri=file://%s',
+                                        'filesrc location=lesson-files/share/fanfare.midi ! wildmidi ! alsasink'),
+                         ('timidity', '-idqq %s'),
+                         ('drvmidi', ''),
+                         ('playmidi', ''),
+             ],
             },
             {'id': 'mp3',
              'testfile': 'fanfare.mp3',
              'label': _("MP3:"),
-             'players': [],
+             'players': [
+                        ('gst-launch', 'playbin uri=file://%s',
+                                       'filesrc location=%s ! mad ! alsasink'),
+                        ('mpg123', ''),
+                        ('alsaplayer', ''),
+             ],
             },
             {'id': 'ogg',
              'testfile': 'fanfare.ogg',
              'label': _("OGG:"),
-             'players': [],
+             'players': [
+                        ('gst-launch', 'playbin uri=file://%s',
+                                       'filesrc location=%s ! oggdemux ! vorbisdec ! audioconvert ! alsasink'),
+                        ('ogg123', ''), 
+                        ('alsaplayer', ''),
+             ],
             },
         ])
-        for d in format_info:
-            for bin, opts in {
-                'wav': (
-                        ('gst-launch', 'playbin uri=file://%s'),
-                        ('gst-launch', 'filesrc location=%s ! wavparse ! alsasink'),
-                        ('play', '%s'),
-                        ('aplay', '%s'),
-                        ('esdplay', '%s')),
-                'midi': (
-                         ('gst-launch', 'playbin uri=file://%s'),
-                         ('gst-launch', 'filesrc location=lesson-files/share/fanfare.midi ! wildmidi ! alsasink'),
-
-                         ('timidity', '-idqq %s'),
-                         ('drvmidi', ''),
-                         ('playmidi', '')),
-                'mp3': (
-                        ('gst-launch', 'playbin uri=file://%s'),
-                        ('gst-launch', 'filesrc location=%s ! mad ! alsasink'),
-                        ('mpg123', ''),
-                        ('alsaplayer', '')),
-                'ogg': (
-                        ('gst-launch', 'playbin uri=file://%s'),
-                        ('gst-launch', 'filesrc location=%s ! oggdemux ! vorbisdec ! audioconvert ! alsasink'),
-                        ('ogg123', ''), 
-                        ('alsaplayer', ''))
-                        }[d['id']]:
-                v = osutils.find_progs((bin,))
-                if v:
-                    d['players'].append((v[0], " ".join((v[0], opts))))
-        self.g_player_boxes = {}
         sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
         for format in format_info:
-            liststore = gtk.ListStore(gobject.TYPE_STRING)
-            for path, cmd in format['players']:
-                liststore.append((cmd,))
-            self.g_player_boxes[format['id']] = gtk.ComboBoxEntry(liststore)
-            self.g_player_boxes[format['id']].child.set_text(cfg.get_string('sound/%s_player' % format['id']))
+            cmd_liststore = gtk.ListStore(gobject.TYPE_STRING)
+            for player_data in format['players']:
+                cmd_liststore.append((player_data[0],))
+            cmd_combo = gtk.ComboBoxEntry(cmd_liststore)
+            cmd_combo.opts = gtk.combo_box_entry_new_text()
+            cmd_combo.child.set_text(cfg.get_string('sound/%s_player' % format['id']))
+            cmd_combo.opts.child.set_text(cfg.get_string('sound/%s_player_options' % format['id']))
+            for s in format['players']:
+                if s[0] == cmd_combo.child.get_text():
+                    for o in s[1:]:
+                        cmd_combo.opts.append_text(o)
+            
             def _changed(widget, formatid):
+                if widget.get_active() != -1:
+                    widget.opts.get_model().clear()
+                    for player_options in format['players'][widget.get_active()][1:]:
+                        widget.opts.append_text(player_options)
+                    widget.opts.child.set_text(format['players'][widget.get_active()][1])
                 self.set_string('sound/%s_player' % formatid,
                                 widget.child.get_text())
-            self.g_player_boxes[format['id']].connect('changed',
-                _changed, format['id'])
-            button = gtk.Button(_("Test"))
-            button.connect('clicked', self.test_XXX_player,
+                binary_found = False
+                if os.path.exists(widget.child.get_text()):
+                    binary_found = True
+                else:
+                    for dir in os.environ['PATH'].split(os.path.pathsep):
+                        if os.path.exists(os.path.join(dir, widget.child.get_text())):
+                            binary_found = True
+                            break
+                widget.testbutton.set_sensitive(binary_found)
+                    
+            def _opts_changed(widget, formatid):
+                self.set_string('sound/%s_player_options' % formatid,
+                                widget.child.get_text())
+            cmd_combo.connect('changed', _changed, format['id'])
+            cmd_combo.opts.connect('changed', _opts_changed, format['id'])
+            testbutton = gtk.Button(_("Test"))
+            testbutton.set_tooltip_text(_("This button is clickable only if the binary is found."))
+            cmd_combo.testbutton = testbutton
+            testbutton.connect('clicked', self.test_XXX_player,
                            format['id'], format['testfile'])
             box = gu.hig_label_widget(format['label'],
-                                      [self.g_player_boxes[format['id']], button],
-                                      sizegroup)
+                                      [cmd_combo, cmd_combo.opts, testbutton],
+                                      sizegroup, True, True)
             category_vbox.pack_start(box)
     def test_XXX_player(self, w, typeid, testfile):
         try:
             soundcard.play_mediafile(typeid, os.path.join('lesson-files/share', testfile))
-        except Exception, e:
-            self.m_app.m_ui.display_exception_message(e)
+        except osutils.BinaryBaseException, e:
+            self.m_app.m_ui.display_error_message2(e.msg1, e.msg2)
     def create_linux_sound_page(self):
         page_vbox = gu.hig_dlg_vbox()
         self.g_notebook.append_page(page_vbox, gtk.Label(_("Sound Setup")))
@@ -523,18 +606,18 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
               c4 e g8 e c4
             }
             """, 130, 0, 100)
-        except Exception, e:
-            if isinstance(e, osutils.OsUtilsException):
-                self.m_app.m_ui.display_exception_message(e)
-            else:
-                raise
+        # Here we are only cathing exceptions we know the MidiFileSynth
+        # can raise. Maybe we should catch something from the Sequencer
+        # synths too?
+        except osutils.BinaryBaseException, e:
+            self.m_app.m_ui.display_error_message2(e.msg1, e.msg2)
     def on_apply(self, *v):
         """Returns -1 if sound init fails."""
         if soundcard.synth:
             soundcard.synth.close()
         if self.g_midiplayer_radio.get_active():
             soundcard.initialise_external_midiplayer(
-                  self.g_player_boxes['midi'].child.get_text())
+                  cfg.get_string("sound/midi_player"))
             soundcard.synth.error_report_cb = self.m_app.m_ui.display_error_message
         elif self.g_device_radio.get_active():
             try:
