@@ -16,6 +16,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import webbrowser
+import textwrap
 # We move x-www-browser to the end of the list because on my
 # debian etch system, the browser does will freeze solfege until
 # I close the browser window.
@@ -35,17 +36,6 @@ import shutil
 
 import winlang
 
-# The sets module was added in python 2.3. So we will use that module
-# if we are running python 2.3, and the builtin set if we are using
-# python 2.4
-# FIXME: remove this code when we require python 2.4
-try:
-    set()
-except NameError:
-    import __builtin__
-    import sets
-    __builtin__.set = sets.Set
-
 import runtime
 
 import buildinfo
@@ -142,6 +132,7 @@ runtime.init(options)
 
 import gtk
 import utils
+import i18n
 
 class SplashWin(gtk.Window):
     def __init__(self):
@@ -230,7 +221,7 @@ class MusicViewerWindow(gtk.Dialog):
         gtk.Dialog.__init__(self)
         self.m_app = app
         self.set_default_size(500, 300)
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer(utils.play_tone)
+        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
         self.vbox.pack_start(self.g_music_displayer)
         b = gu.bButton(self.action_area, _("Close"), self.m_app.close_musicviewer)
         b.grab_focus()
@@ -721,6 +712,11 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 os.rmdir(os.path.join(os.path.join(filesystem.app_data(), "learningtrees")))
         item = self.g_ui_manager.get_widget("/Menubar/FileMenu/LearningTreesMenu")
         item.connect('activate', lambda s: self.create_learning_trees_menu())
+        try:
+            i18n.locale_setup_failed
+            print >> sys.stderr, "\n".join(textwrap.wrap("Translations are disabled because your locale settings are broken. This is not a bug in GNU Solfege, so don't report it. The README file distributed with the program has some more details."))
+        except AttributeError:
+            pass
     def display_discarded_lessonfiles_info(self):
         assert self.m_app.lessonfile_manager.m_discards
         dlg = gtk.Dialog(_("Discarded lesson files"), buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
