@@ -152,13 +152,9 @@ class Gui(abstract.IntervalGui):
             i = mpd.Interval()
             i.set_from_int(interval)
             n = self.m_t.m_tonika + i
-            track = soundcard.Track()
-            track.set_bpm(self.get_int('config/default_bpm'))
-            track.set_patch(self.get_int('config/preferred_instrument'))
-            track.note(4, self.m_t.m_tonika.semitone_pitch(),
-                    self.get_int('config/preferred_instrument_velocity'))
-            track.note(4, n.semitone_pitch(),
-                    self.get_int('config/preferred_instrument_velocity'))
+            track = utils.new_track()
+            track.note(4, self.m_t.m_tonika.semitone_pitch())
+            track.note(4, n.semitone_pitch())
             soundcard.synth.play_track(track)
             return
         if mouse_button != 1:
@@ -174,9 +170,7 @@ class Gui(abstract.IntervalGui):
         if midi_int:
             # midi_int is only set when we use some of the instrument widgets,
             # not when we use the buttons interface.
-            soundcard.play_note(self.get_int('config/preferred_instrument'),
-                        4, midi_int,
-                        self.get_int('config/preferred_instrument_velocity'))
+            utils.play_note(4, midi_int)
         if self.m_t.q_status == self.QSTATUS_GIVE_UP:
             return
         if self.m_t.q_status == self.QSTATUS_SOLVED:
